import ctypes
import library_singleton

OPEN_ATTRIBUTE_NO_DOWNLOAD_IF_PROGRAMMED = 0x2000000    # 1 << 25
OPEN_ATTRIBUTE_NO_SIGNATURE_CHECK = 0x80000000          # 1 << 31


class Session(object):
    def __init__(self, resource, bitfile, **kwargs):
        self._library = library_singleton.get()
        self.resource = resource
        self._session = ctypes.c_uint32()
        err = self._library.niFlexRIO_OpenSession(
            self._create_string_buffer(resource),
            self._create_string_buffer(bitfile),
            OPEN_ATTRIBUTE_NO_DOWNLOAD_IF_PROGRAMMED | OPEN_ATTRIBUTE_NO_SIGNATURE_CHECK,
            ctypes.pointer(self._session))
        self.handle_error(err)

    def _create_string_buffer(self, string):
        return ctypes.create_string_buffer(string.encode('utf-8'))

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        try:
            self.close()
        except Exception:
            pass

    def _get_error(self, error_code):
        size = 4096
        buf = (ctypes.c_char * size)()
        self._library.niFlexRIO_GetError(
            self._session,
            error_code,
            size,
            buf)
        return buf.value.decode('utf-8')

    def handle_error(self, error_code):
        if error_code < 0:
            raise IOError(error_code, self._get_error(error_code))

    def close(self, reset_if_last_session=True):
        self._library.niFlexRIO_CloseSession(self._session, reset_if_last_session)

    def read_io_module_info(self):
        product_id = ctypes.c_uint16()
        vendor_id = ctypes.c_uint16()
        serial_number = ctypes.c_uint32()
        part_number = ctypes.c_uint32()
        variant_info = ctypes.c_uint8()
        hardware_revision = ctypes.c_uint16()
        err = self._library.niFlexRIOMDK_readIoModuleInfo(
            self._session,
            ctypes.pointer(product_id),
            ctypes.pointer(vendor_id),
            ctypes.pointer(serial_number),
            ctypes.pointer(part_number),
            ctypes.pointer(variant_info),
            ctypes.pointer(hardware_revision))
        self.handle_error(err)
        info = {}
        info["product_id"] = product_id.value
        info["vendor_id"] = vendor_id.value
        info["serial_number"] = serial_number.value
        info["part_number"] = part_number.value
        info["variant_info"] = variant_info.value
        info["hardware_revision"] = hardware_revision.value

        return info

    def write_i2c(self, bus_instance, slave_addr, data, repeated_start=False):
        num_bytes_to_write = len(data)
        buf = ctypes.c_uint8 * num_bytes_to_write
        buf = buf.from_buffer(data)
        rep_start = ctypes.c_uint8(1 if repeated_start else 0)
        bytes_written = ctypes.c_size_t()

        err = self._library.niFlexRIO_WriteI2C(
            self._session,
            self._create_string_buffer(bus_instance),
            ctypes.c_uint16(slave_addr),
            buf,
            num_bytes_to_write,
            rep_start,
            ctypes.pointer(bytes_written))

        self.handle_error(err)

    def transfer_i2c(self, bus_instance, slave_addr, write_data, num_bytes_to_read, repeated_start=False):
        num_bytes_to_write = len(write_data)
        write_buf = ctypes.c_uint8 * num_bytes_to_write
        write_buf = write_buf.from_buffer(write_data)
        read_buf = (ctypes.c_uint8 * num_bytes_to_read)()
        rep_start = ctypes.c_uint8(1 if repeated_start else 0)
        bytes_written = ctypes.c_size_t()
        bytes_read = ctypes.c_size_t()

        err = self._library.niFlexRIO_TransferI2C(
            self._session,
            self._create_string_buffer(bus_instance),
            ctypes.c_uint16(slave_addr),
            write_buf,
            num_bytes_to_write,
            read_buf,
            num_bytes_to_read,
            rep_start,
            ctypes.pointer(bytes_written),
            ctypes.pointer(bytes_read))

        self.handle_error(err)

        return bytearray(read_buf)
