from flexrio import session
import posc_binary
import re
import simulation
import struct
import sys
import time


class ProductId(object):
    PXIE_1486_01 = 0x7A82
    PXIE_1486_02 = 0x7A83
    PXIE_1486_03 = 0x7A84
    PXIE_1487_01 = 0x7A85
    PXIE_1487_02 = 0x7A86
    PXIE_1487_03 = 0x7A87


class ModelName(object):
    PXIE_1486 = 1486
    PXIE_1487 = 1487

    @classmethod
    def models(cls):
        return [
            cls.PXIE_1486,
            cls.PXIE_1487]


class SerDesVendor(object):
    TI = 0
    MAXIM = 1


class SerDesType(object):
    SER = 0
    DES = 1


class Mode(object):
    SIMULATION = 0          # Calls Simulation "dummy" functions
    RUNTIMECONFIG = 1       # Calls Session FlexRIO Session functions to program hardware
    POSC = 2                # Calls PoscBinary functions to convert commands to POSC binary script


class FlexRIOWrapper(object):
    def __init__(self, resource, bitfile, serdes, simulated_module, script, posc, verbose=False):
        self.resource = resource
        self.bitfile = bitfile
        self.serdes_name = serdes
        self.verbose = verbose
        self.command_executed = False
        self.orig_sleep = time.sleep
        self.posc_file = posc

        if simulated_module != 0:
            self.mode = Mode.SIMULATION
        elif self.posc_file:
            self.mode = Mode.POSC
        else:
            self.mode = Mode.RUNTIMECONFIG

        if self.serdes_name.startswith('Ser'):
            self.serdes_type = SerDesType.SER
        elif self.serdes_name.startswith('Des'):
            self.serdes_type = SerDesType.DES
        else:
            raise ValueError('serdes name must start with Ser or Des')

        pid = 0
        if self.mode == Mode.SIMULATION:
            # argparse enforces that simulated_module can only be 1486 or 1487
            self.serdes_vendor = SerDesVendor.TI if simulated_module == ModelName.PXIE_1486 else SerDesVendor.MAXIM
            self.session = simulation.Simulation()
        elif self.mode == Mode.POSC:
            self.session = posc_binary.PoscBinary(self.posc_file)
            if script.endswith('.py'):
                self.serdes_vendor = SerDesVendor.TI
            elif script.endswith('.cpp'):
                self.serdes_vendor = SerDesVendor.MAXIM
            else:
                raise ValueError('Incompatible file type: .py (TI) or .cpp (Maxim) expected')
        elif self.mode == Mode.RUNTIMECONFIG:
            self.session = session.Session(bitfile=bitfile, resource=resource)
            info = self.session.read_io_module_info()
            pid = info['product_id']
            if pid in [ProductId.PXIE_1486_01, ProductId.PXIE_1486_02, ProductId.PXIE_1486_03]:
                self.serdes_vendor = SerDesVendor.TI
            elif pid in [ProductId.PXIE_1487_01, ProductId.PXIE_1487_02, ProductId.PXIE_1487_03]:
                self.serdes_vendor = SerDesVendor.MAXIM
            else:
                raise ValueError('module must be NI PXIe-1486 or NI PXIe-1487', pid)
        else:
            raise ValueError('Invalid mode')

    def write_i2c(self, slave_addr, bytes_to_write):
        self.command_executed = True
        self.session.write_i2c(self.serdes_name, slave_addr, bytes_to_write)
        if self.verbose:
            print 'write_i2c(resource = {}, serdes = {}, addr = {}, data = {})'.format(
                self.resource,
                self.serdes_name,
                hex(slave_addr),
                ', '.join(hex(x) for x in bytes_to_write))

    def transfer_i2c(self, slave_addr, bytes_to_write, num_bytes_to_read):
        self.command_executed = True
        bytes_read = self.session.transfer_i2c(self.serdes_name, slave_addr, bytes_to_write, num_bytes_to_read)
        result = int(bytes_read[0]) if len(bytes_read) == 1 else [int(x) for x in bytes_read]
        if self.verbose:
            print 'transfer_i2c(resource = {}, serdes = {}, addr = {}, bytes_to_write = {}, num_bytes_to_read = {}) = {}'.format(
                self.resource,
                self.serdes_name,
                hex(slave_addr),
                ', '.join(hex(x) for x in bytes_to_write),
                num_bytes_to_read,
                [hex(x) for x in result] if isinstance(result, list) else hex(result))
        return result

    def sleep(self, seconds):
        if self.mode == Mode.RUNTIMECONFIG:
            self.orig_sleep(seconds)
        else:
            self.session.sleep(seconds)

    def pre_execute(self, script):
        if (self.verbose):
            print 'running the script {}....'.format(script)
        if self.serdes_vendor == SerDesVendor.TI and (self.mode == Mode.SIMULATION or self.mode == Mode.POSC):
            # override time.sleep so that we can not sleep in simulation mode or write out a sleep command in posc mode
            time.sleep = self.sleep

    def post_execute(self):
        if self.serdes_vendor == SerDesVendor.TI and (self.mode == Mode.SIMULATION or self.mode == Mode.POSC):
            # restore time.sleep to standard function
            time.sleep = self.orig_sleep

        if not self.command_executed:
            raise ValueError('script did not contain any valid commands')

        if self.mode == Mode.POSC:
            self.session.end_of_script()
