import os
import re


# Maxim's script format is a comma delimited list of the following hex values:
# [number of bytes left in the command] [I2C slave address] [I2C register address] [data byte 0] [data byte 1]...
# If there are two values and the first is 0, that's a special indication that we should delay for the number of milliseconds in the second byte
def parse_line(fw, line):
    # skip empty lines
    if len(line) == 0:
        return
    # look for comma separated list of hex byte values
    pattern = re.compile(r'0x([0-9a-fA-F]{1,2}),')
    match = [int(numeric_string, 16) for numeric_string in pattern.findall(line)]
    # need at least two hex values to do anything meaningful, skip anything else
    if len(match) == 0:
        # if there were no hex byte matches at all, it's probably a #include, a declaration, etc., skip silently
        return
    elif len(match) == 1:
        # this seems to be a special value to indicate the end of a set of commands, skip silently
        if match[0] == 0xFF:
            return
        else:
            # if there was at least one value that matched, this may have been an attempt at a command, throw an error
            raise ValueError('a minimum of two comma-separated hex values is required for a command')
    byte_count = match[0]
    if byte_count == 0:
        # special command for delay, value is ms
        delay_seconds = match[1] / 1000.0
        if (fw.verbose):
            print '{} second delay'.format(delay_seconds)
        fw.sleep(delay_seconds)
        return
    if (byte_count + 1) != len(match):
        raise ValueError('mismatch between number of words value with actual number of words')
    if byte_count < 2:
        raise ValueError('cannot perform read or write command with less than 2 byte values specified')
    dev_addr = match[1]
    # includes reg addr and then any additional bytes to write
    bytes_to_write = bytearray(match[2:])
    # even number for dev_addr indicates a write
    if dev_addr % 2 == 0:
        fw.write_i2c(dev_addr, bytes_to_write)
    else:
        dev_addr -= 1  # clear the bit indicating read to pass the correct dev_addr
        num_bytes_to_read = bytes_to_write.pop()
        result = fw.transfer_i2c(dev_addr, bytes_to_write, num_bytes_to_read)


def execute_script(fw, script):
    with open(script, 'r') as script_file:
        contents = script_file.read()
        # remove all spaces
        contents = contents.replace(' ', '')
        # remove all tabs
        contents = contents.replace('\t', '')
        # remove anything between /* */
        contents = re.sub(re.compile('/\\*.*?\\*/', re.DOTALL), '', contents)
        # remove anything after //
        contents = re.sub(re.compile('//.*?$', re.MULTILINE), '', contents)
        line_num = 0
        for line in contents.splitlines():
            try:
                parse_line(fw, line)
            except IOError as ioe:
                # augment errors from the FlexRIO driver with information about where things went wrong
                message = ioe.strerror + '\n{}, line {}: {}'.format(os.path.basename(script_file.name), line_num, line)
                raise IOError(ioe.errno, message)
            except Exception as e:
                raise type(e)('{}\n{}, line {}: {}'.format(e.args, os.path.basename(script_file.name), line_num, line))
            line_num += 1
